<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 3/18/16
 * Time: 5:43 PM
 */

namespace qfq;


class BodytextParser {

    /**
     * @param $bodytext
     * @return mixed
     */
    public function process($bodytext) {
        $bodytext = $this->stripAndRemoveComment($bodytext);
        $bodytext = $this->joinLine($bodytext);
        $bodytext = $this->unNest($bodytext);
        $bodytext = $this->stripAndRemoveComment($bodytext);

        return $bodytext;

    }

    /**
     * Trim all lines, remove all empty lines and  all lines which start with '#'
     * @param $bodytext
     * @return string
     */

    private function stripAndRemoveComment($bodytext) {
        $data = array();

        $src = explode(PHP_EOL, $bodytext);
        foreach ($src as $row) {
            $row = trim($row);
            if ($row === '' || $row[0] === '#') {
                continue;
            }
            $data[] = $row;
        }

        return implode(PHP_EOL, $data);
    }

//PREG_SPLIT_DELIM_CAPTURE

    /**
     * Join lines, which do not begin with '<level>.<keyword>[ ]='
     *
     * @param array $bodytextArray
     * @return string
     */
    private function joinLine($bodytext) {
        $data = array();
        $bodytextArray = explode(PHP_EOL, $bodytext);

        $full = '';
        foreach ($bodytextArray as $row) {
            // Valid 'new line' starts indicators: form, <level>, <level.sublevel>, <level>.<keyword>, {, <level> {, }
            if ((1 === preg_match('/^\s*(\d*(\.)?)*\s*(head|althead|tail|sql|rbeg|rend|renr|rsep|fbeg|fend|fsep|form) *=/', $row))
                || (1 === preg_match('/^\s*(\d*(\.)?)*\s*({|})\s*/', $row))
                || (1 === preg_match('/^\s*(\d+(\.)?)+/', $row))
            ) {

                // if there is already something: save this
                if ($full !== '')
                    $data[] = $full;

                // start new line
                $full = $row;

            } else {
                // continue row: concat
                $full .= ' ' . $row;
            }
        }

        // Save last line
        if ($full !== '')
            $data[] = $full;

        return implode(PHP_EOL, $data);
    }

    private function unNest($bodytext) {
        // Replace '\{' | '\}' by internal token. All remaining '}' | '{' means: 'nested'
        $bodytext = str_replace('\{', '#&[_#', $bodytext);
        $bodytext = str_replace('\}', '#&]_#', $bodytext);
        $bodytext = str_replace('{{', '#&[[_#', $bodytext);
        $bodytext = str_replace('}}', '#&]]_#', $bodytext);

        $result = $bodytext;
        $posFirstClose = strpos($result, '}');

        while ($posFirstClose !== false) {
            $posMatchOpen = strrpos(substr($result, 0, $posFirstClose), '{');

            if ($posMatchOpen === false) {
                throw new \qfq\UserException("Missing open delimiter: $result", ERROR_MISSING_OPEN_DELIMITER);
            }

            $pre = substr($result, 0, $posMatchOpen);
            if ($pre === false)
                $pre = '';

            $post = substr($result, $posFirstClose + 1);
            if ($post === false)
                $post = '';

            // trim also removes '\n'
            $match = trim(substr($result, $posMatchOpen + 1, $posFirstClose - $posMatchOpen - 1));

            // "10.sql = SELECT...\n20 {\n
            $levelStartPos = strrpos(trim($pre), PHP_EOL);
            $levelStartPos = ($levelStartPos === false) ? 0 : $levelStartPos + 1;  // Skip PHP_EOL

            $level = trim(substr($pre, $levelStartPos));
//            if($level==='') {
//                $pre=
//            }
            // remove 'level' from last line
            $pre = substr($pre, 0, $levelStartPos);

            // Split nested content in single rows
            $lines = explode(PHP_EOL, $match);
            foreach ($lines as $line) {
                $pre .= $level . '.' . $line . PHP_EOL;
            }

            $result = $pre . $post;
            $posFirstClose = strpos($result, '}');
        }

        $result = str_replace('#&[_#', '{', $result);
        $result = str_replace('#&]_#', '}', $result);
        $result = str_replace('#&[[_#', '{{', $result);
        $result = str_replace('#&]]_#', '}}', $result);


        return $result;
    }

}